#include "types.h"
#include "socket.h"
#include "utils.h"


int log_init(int handle, unsigned int ip)
{
	int log_socket = socket(handle, AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (log_socket < 0)
		return -1;

	struct sockaddr_in connect_addr;
	memset(&connect_addr, 0, sizeof(connect_addr));
	connect_addr.sin_family = AF_INET;
	connect_addr.sin_port = 4405;
	connect_addr.sin_addr.s_addr = ip;

	if(connect(handle, log_socket, (struct sockaddr*)&connect_addr, sizeof(connect_addr)) < 0)
	{
	    closesocket(handle, log_socket);
	    log_socket = -1;
	}

	return log_socket;
}

void log_print(int handle, int log_socket, const char *str)
{
    // socket is always 0 initially as it is in the BSS
    if(log_socket <= 0) {
        return;
    }


    int len = strlen(str);
    int ret;
    while (len > 0) {
        int block = len < 1400 ? len : 1400; // take max 1400 bytes per UDP packet
        ret = send(handle, log_socket, str, block, 0);
        if(ret < 0)
            break;

        len -= ret;
        str += ret;
    }
}

void log_deinit(int handle, int log_socket)
{
    closesocket(handle, log_socket);
}
