#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "socket.h"
#include "svc.h"

int socketInit()
{
	int ret = svcOpen("/dev/socket", 0);

	if(ret > 0)
	{
		return ret;
	}

	return ret;
}

int socketExit(int socket_handle)
{
	int ret = svcClose(socket_handle);

	socket_handle = 0;

	return ret;
}

static void* allocIobuf(u32 size)
{
	void* ptr = svcAlloc(0xCAFF, size);

	if(ptr) memset(ptr, 0x00, size);

	return ptr;
}

static void freeIobuf(void* ptr)
{
	svcFree(0xCAFF, ptr);
}

int	socket(int socket_handle, int domain, int type, int protocol)
{
	u8* iobuf = allocIobuf(0xC);
	u32* inbuf = (u32*)iobuf;

	inbuf[0] = domain;
	inbuf[1] = type;
	inbuf[2] = protocol;

	int ret = svcIoctl(socket_handle, 0x11, inbuf, 0xC, NULL, 0);

	freeIobuf(iobuf);
	return ret;
}

int	closesocket(int socket_handle, int sockfd)
{
	u8* iobuf = allocIobuf(0x4);
	u32* inbuf = (u32*)iobuf;

	inbuf[0] = sockfd;

	int ret = svcIoctl(socket_handle, 0x3, inbuf, 0x4, NULL, 0);

	freeIobuf(iobuf);
	return ret;
}

int	bind(int socket_handle, int sockfd, const struct sockaddr *addr, socklen_t addrlen)
{
	if(addrlen != 0x10) return -1;

	u8* iobuf = allocIobuf(0x18);
	u32* inbuf = (u32*)iobuf;

	inbuf[0] = sockfd;
	memcpy(&inbuf[1], addr, addrlen);
	inbuf[5] = addrlen;

	int ret = svcIoctl(socket_handle, 0x2, inbuf, 0x18, NULL, 0);

	freeIobuf(iobuf);
	return ret;
}

int	connect(int socket_handle, int sockfd, const struct sockaddr *addr, socklen_t addrlen)
{
	if(addrlen != 0x10) return -1;

	u8* iobuf = allocIobuf(0x18);
	u32* inbuf = (u32*)iobuf;

	inbuf[0] = sockfd;
	memcpy(&inbuf[1], addr, addrlen);
	inbuf[5] = addrlen;

	int ret = svcIoctl(socket_handle, 0x4, inbuf, 0x18, NULL, 0);

	freeIobuf(iobuf);
	return ret;
}

int send(int socket_handle, int sockfd, const void *buf, size_t len, int flags)
{
	if(!buf || !len) return -101;

	// TODO : size checks, split up data into multiple vectors if necessary
	void* data_buf = svcAllocAlign(0xCAFF, len, 0x40);
	if(!data_buf) return -100;

	u8* iobuf = allocIobuf(0x38);
	iovec_s* iovec = (iovec_s*)iobuf;
	u32* inbuf = (u32*)&iobuf[0x30];

	memcpy(data_buf, buf, len);

	inbuf[0] = sockfd;
	inbuf[1] = flags;

	iovec[0].ptr = inbuf;
	iovec[0].len = 0x8;
	iovec[1].ptr = (void*)data_buf;
	iovec[1].len = len;

	int ret = svcIoctlv(socket_handle, 0xE, 4, 0, iovec);

	freeIobuf(data_buf);
	freeIobuf(iobuf);
	return ret;
}
